unit About_U;
// Description: About Dialog
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,
  OTFEFreeOTFE_U, SDUStdCtrls, SDUForms;

type
  TAbout_F = class(TSDUForm)
    pbOK: TButton;
    Image1: TImage;
    lblAppID: TLabel;
    lblDescription: TLabel;
    Label2: TLabel;
    lblBeta: TLabel;
    lblDriverVersion: TLabel;
    Label4: TLabel;
    pnlDividerUpper: TPanel;
    pnlDividerLower: TPanel;
    Label3: TLabel;
    SDUURLLabel1: TSDUURLLabel;
    procedure FormShow(Sender: TObject);
  private
  public
    OTFEFreeOTFE: TOTFEFreeOTFE;
  end;

implementation

{$R *.DFM}

uses
  ShellApi,  // Needed for ShellExecute
  SDUi18n,
  SDUGeneral,
  FreeOTFEConsts;

procedure TAbout_F.FormShow(Sender: TObject);
const
  CONTROL_MARGIN = 10;
var
  majorVersion   : integer;
  minorVersion   : integer;
  revisionVersion: integer;
  buildVersion   : integer;
  OTFEVersion: string;
  descAdjustDown: integer;
begin
  SDUGetVersionInfo('', majorVersion, minorVersion, revisionVersion, buildVersion);
  lblAppID.caption := 'v'+SDUGetVersionInfoString('');
  if APP_BETA_BUILD>-1 then
    begin
    lblAppID.caption := lblAppID.caption + ' BETA '+inttostr(APP_BETA_BUILD);
    end;

  lblBeta.visible := (APP_BETA_BUILD>-1);


  if OTFEFreeOTFE.Active then
    begin
    OTFEVersion := OTFEFreeOTFE.VersionStr();
    if (OTFEVersion<>'') then
      begin
      OTFEVersion := SDUParamSubstitute(_('FreeOTFE driver: %1'), [OTFEVersion]);
      end;
    end
  else
    begin
    OTFEVersion := _('The main FreeOTFE driver is either not installed, or not started');
    end;


  // Some translated languages may increase the number of lines the
  // description takes up. Here we increase the height of the dialog to
  // compensate, and nudge the controls below it down
  descAdjustDown := (
                     lblDescription.Top +
                     lblDescription.Height +
                     CONTROL_MARGIN
                    ) - pnlDividerUpper.Top;

  self.height := self.height + descAdjustDown;

  pnlDividerUpper.Top  := pnlDividerUpper.Top + descAdjustDown;
  SDUURLLabel1.Top     := SDUURLLabel1.Top + descAdjustDown;
  pnlDividerLower.Top  := pnlDividerLower.Top + descAdjustDown;
  lblDriverVersion.Top := lblDriverVersion.Top + descAdjustDown;
  pbOK.Top             := pbOK.Top + descAdjustDown;


  SDUCenterControl(lblDescription, ccHorizontal);

  lblDriverVersion.caption := OTFEVersion;
  SDUCenterControl(lblDriverVersion, ccHorizontal);

  pnlDividerUpper.Caption := '';
  pnlDividerLower.Caption := '';

end;

END.

