unit FreeOTFESettings;
// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Classes, // Required for TShortCut
  INIFiles,
  SDUMRUList;

{$IFDEF _NEVER_DEFINED}
// This is just a dummy const to fool dxGetText when extracting message
// information
// This const is never used; it's #ifdef'd out - SDUCRLF in the code refers to
// picks up SDUGeneral.SDUCRLF
const
  SDUCRLF = ''#13#10;
{$ENDIF}
  
type
  TDragDropFileType = (ftPrompt, ftFreeOTFE, ftLinux);

  TSettingsSaveLocation = (slNone, slExeDir, slProfile, slRegistry);

resourcestring
  DRAGDROPFILETYPE_PROMPT   = '<prompt user>';
  DRAGDROPFILETYPE_FREEOTFE = 'FreeOTFE';
  DRAGDROPFILETYPE_LINUX    = 'Linux';

  SAVELOCATION_DO_NOT_SAVE  = '<Do not save settings>';
  SAVELOCATION_EXE_DIR      = 'File in FreeOTFE directory';
  SAVELOCATION_USER_PROFILE = 'File in user profile';
  SAVELOCATION_REGISTRY     = 'Windows registry';

  UNKNOWN_SETTINGS_LOCATION = 'Unknown settings location?! Please report seeing this message!';

const
  REGISTRY_SETTINGS_LOCATION = '\Software\FreeOTFE';
  
  DragDropFileTypeTitlePtr: array [TDragDropFileType] of Pointer = (
                                  @DRAGDROPFILETYPE_PROMPT,
                                  @DRAGDROPFILETYPE_FREEOTFE,
                                  @DRAGDROPFILETYPE_LINUX
                                 );

  SettingsSaveLocationSearchOrder: array [TSettingsSaveLocation] of TSettingsSaveLocation = (
                                  slExeDir,
                                  slProfile,
                                  slRegistry,
                                  slNone
                                 );

type
  TFreeOTFESettings = class
  protected
    function IdentifyWhereSettingsStored(): TSettingsSaveLocation;

    procedure _Load(iniFile: TCustomINIFile);
    function  _Save(iniFile: TCustomINIFile): boolean;

    function  GetSettingsObj(): TCustomIniFile;
  public
    OptSettingsVersion: integer;
    OptSaveSettings: TSettingsSaveLocation;

    // General...
    OptExploreAfterMount: boolean;
    OptAllowMultipleInstances: boolean;
    OptDisplayToolbar: boolean;
    OptDisplayToolbarLarge: boolean;
    OptDisplayToolbarCaptions: boolean;
    OptDisplayStatusbar: boolean;
    OptAutoStartPortable: boolean;
    OptAdvancedMountDlg: boolean;
    OptDragDropFileType: TDragDropFileType;
    OptRevertVolTimestamps: boolean;
    OptDefaultDriveLetter: char;
    OptShowPasswords: boolean;
    OptLanguageCode: string;

    // Prompts and messages
    OptPromptMountSuccessful: boolean;  // If set, display an info msg after successful mount
    OptPromptDismountOnClose: boolean;  // If set, prompt the user if they want to dismount any mounted volumes when closing
    OptPromptDriverShutdownOnClose: boolean;  // If set, prompt the user if they want to shutdown any portable mode drivers when closing

    // System tray icon...
    OptSystemTrayIconDisplay: boolean;
    OptSystemTrayIconMinTo: boolean;
    OptSystemTrayIconCloseTo: boolean;

    // Hotkey...
    OptHKeyEnableDismount: boolean;
    OptHKeyKeyDismount: TShortCut;
    OptHKeyEnableDismountEmerg: boolean;
    OptHKeyKeyDismountEmerg: TShortCut;

    // Autorun...
    OptPostMountExe: string;
    OptPreDismountExe: string;
    OptPostDismountExe: string;
    OptPrePostExeWarn: boolean;

    // PKCS11...
    OptPKCS11Enable: boolean;
    OptPKCS11Library: string;
    OptPKCS11AutoMount: boolean;
    OptPKCS11AutoMountVolume: string;
    OptPKCS11AutoDismount: boolean;

    // MRU list...
    OptMRUList: TSDUMRUList;

    constructor Create();
    destructor Destroy(); override;

    procedure Load();
    function  Save(): boolean;

    procedure Assign(copyFrom: TFreeOTFESettings);

    function  DestroySettingsFile(loc: TSettingsSaveLocation): boolean;
    function  GetSettingsFilename(loc: TSettingsSaveLocation): string;

    function  PrettyPrintSettingsFile(loc: TSettingsSaveLocation): string;

  end;

  function DragDropFileTypeTitle(dragDropfileType: TDragDropFileType): string;

var
  // Global variable
  Settings: TFreeOTFESettings;

implementation

uses
  Windows,  // Required to get rid of compiler hint re DeleteFile
  SysUtils,  // Required for ChangeFileExt, DeleteFile
  Registry,
  Dialogs,
  SDUi18n,
  SDUGeneral,
  SDUDialogs,
  Menus,  // Required for ShortCutToText and TextToShortCut
  ShlObj;  // Required for CSIDL_PERSONAL

const
  SETTINGS_V1 = 1;
  SETTINGS_V2 = 2;

  // -- General section --
  SECTION_GENERAL = 'General';
    OPT_SETTINGSVERSION                          = 'SettingsVersion';
      DFLT_OPT_SETTINGSVERSION                      = SETTINGS_V2;
    OPT_SAVESETTINGS                             = 'SaveSettings';
      DFLT_OPT_SAVESETTINGS                         = slProfile;
    OPT_EXPLOREAFTERMOUNT                        = 'ExploreAfterMount';
      DFLT_OPT_EXPLOREAFTERMOUNT                    = FALSE;
    OPT_ALLOWMULTIPLEINSTANCES                   = 'AllowMultipleInstances';
      DFLT_OPT_ALLOWMULTIPLEINSTANCES               = FALSE;
    OPT_DISPLAYTOOLBAR                           = 'DisplayToolbar';
      DFLT_OPT_DISPLAYTOOLBAR                       = TRUE;
    OPT_DISPLAYTOOLBARLARGE                      = 'DisplayToolbarLarge';
      DFLT_OPT_DISPLAYTOOLBARLARGE                  = TRUE;
    OPT_DISPLAYTOOLBARCAPTIONS                   = 'DisplayToolbarCaptions';
      DFLT_OPT_DISPLAYTOOLBARCAPTIONS               = TRUE;
    OPT_DISPLAYSTATUSBAR                         = 'DisplayStatusbar';
      DFLT_OPT_DISPLAYSTATUSBAR                     = TRUE;
    OPT_AUTOSTARTPORTABLE                        = 'AutoStartPortable';
      DFLT_OPT_AUTOSTARTPORTABLE                    = FALSE;
    OPT_ADVANCEDMOUNTDLG                         = 'AdvancedMountDlg';
      DFLT_OPT_ADVANCEDMOUNTDLG                     = FALSE;
    OPT_DRAGDROP                                 = 'DragDropFileType';
      DFLT_OPT_DRAGDROP                             = ord(ftFreeOTFE);
    OPT_REVERTVOLTIMESTAMPS                      = 'RevertVolTimestamps';
      DFLT_OPT_REVERTVOLTIMESTAMPS                  = TRUE;
    OPT_DEFAULTDRIVELETTER                       = 'DefaultDriveLetter';
      DFLT_OPT_DEFAULTDRIVELETTER                   = '#';
    OPT_SHOWPASSWORDS                            = 'ShowPasswords';
      DFLT_OPT_SHOWPASSWORDS                        = FALSE;
    OPT_LANGUAGECODE                             = 'LanguageCode';
      DFLT_OPT_LANGUAGECODE                         = '';

  // -- Prompts and messages --
  SECTION_CONFIRMATION = 'Confirmation';
    OPT_PROMPTMOUNTSUCCESSFUL                    = 'OptPromptMountSuccessful';
      DFLT_OPT_PROMPTMOUNTSUCCESSFUL                = TRUE;
    OPT_PROMPTDISMOUNTONCLOSE                    = 'OptPromptDismountOnClose';
      DFLT_OPT_PROMPTDISMOUNTONCLOSE                = TRUE;
    OPT_PROMPTDRIVERSHUTDOWNONCLOSE              = 'OptPromptDriverShutdownOnClose';
      DFLT_OPT_PROMPTDRIVERSHUTDOWNONCLOSE          = TRUE;

  // -- System tray icon section --
  SECTION_SYSTEMTRAYICON = 'SystemTrayIcon';
    OPT_SYSTEMTRAYICONDISPLAY                    = 'Display';
      DFLT_OPT_SYSTEMTRAYICONDISPLAY                = TRUE;
    OPT_SYSTEMTRAYICONMINTO                      = 'MinTo';
      DFLT_OPT_SYSTEMTRAYICONMINTO                  = FALSE;
    OPT_SYSTEMTRAYICONCLOSETO                    = 'CloseTo';
      DFLT_OPT_SYSTEMTRAYICONCLOSETO                = FALSE;

  // -- Hotkey section --
  SECTION_HOTKEY = 'Hotkeys';
    OPT_HOTKEYENABLEDISMOUNT                     = 'EnableDismount';
      DFLT_OPT_HOTKEYENABLEDISMOUNT                 = FALSE;
    OPT_HOTKEYKEYDISMOUNT                        = 'KeyDismount';
      DFLT_OPT_HOTKEYKEYDISMOUNT                    = 'Shift+Ctrl+D';
    OPT_HOTKEYENABLEDISMOUNTEMERG                = 'EnableDismountEmerg';
      DFLT_OPT_HOTKEYENABLEDISMOUNTEMERG            = FALSE;
    OPT_HOTKEYKEYDISMOUNTEMERG                   = 'KeyDismountEmerg';
      DFLT_OPT_HOTKEYKEYDISMOUNTEMERG               = 'Shift+Ctrl+Alt+D';

  // -- Autorun section --
  SECTION_AUTORUN = 'Autorun';
    OPT_POSTMOUNTEXE                             = 'PostMountExe';
      DFLT_OPT_POSTMOUNTEXE                         = '';
    OPT_PREDISMOUNTEXE                           = 'PreDismountExe';
      DFLT_OPT_PREDISMOUNTEXE                       = '';
    OPT_POSTDISMOUNTEXE                          = 'PostDismountExe';
      DFLT_OPT_POSTDISMOUNTEXE                      = '';
    OPT_PREPOSTEXEWARN                           = 'PrePostExeWarn';
      DFLT_OPT_PREPOSTEXEWARN                       = TRUE;

  // -- PKCS#11 section --
  SECTION_PKCS11 = 'PKCS11';
    OPT_PKCS11ENABLE                             = 'Enabled';
      DFLT_OPT_PKCS11ENABLE                        = FALSE;
    OPT_PKCS11LIBRARY                            = 'Library';
      DFLT_OPT_PKCS11LIBRARY                       = '';
    OPT_PKCS11AUTOMOUNT                          = 'AutoMount';
      DFLT_OPT_PKCS11AUTOMOUNT                     = FALSE;
    OPT_PKCS11AUTOMOUNTVOLUME                    = 'AutoMountVolume';
      DFLT_OPT_PKCS11AUTOMOUNTVOLUME               = '';
    OPT_PKCS11AUTODISMOUNT                       = 'AutoDismount';
      DFLT_OPT_PKCS11AUTODISMOUNT                  = FALSE;

  // -- MRU list section --
  SECTION_MRULIST = 'MRUList';
    // (No name for this one)
      DFLT_OPT_MRUMAXITEMS                       = 0;


function DragDropFileTypeTitle(dragDropfileType: TDragDropFileType): string;
begin
  Result := LoadResString(DragDropFileTypeTitlePtr[dragDropfileType]);
end;

constructor TFreeOTFESettings.Create();
begin
  inherited;

  OptMRUList:= TSDUMRUList.Create();

  Load();

end;

destructor TFreeOTFESettings.Destroy();
begin
  OptMRUList.Free();
  
  inherited;
end;

procedure TFreeOTFESettings.Load();
var
  iniFile: TCustomINIFile;
begin
  OptSaveSettings:= IdentifyWhereSettingsStored();
  iniFile := GetSettingsObj();
  try
    _Load(iniFile);
  finally
    iniFile.Free();
  end;

end;

procedure TFreeOTFESettings._Load(iniFile: TCustomINIFile);
var
  useDefaultDriveLetter: string;
begin
  OptSettingsVersion := iniFile.ReadInteger(SECTION_GENERAL, OPT_SETTINGSVERSION, SETTINGS_V1);

  OptExploreAfterMount       := iniFile.ReadBool(SECTION_GENERAL,   OPT_EXPLOREAFTERMOUNT,      DFLT_OPT_EXPLOREAFTERMOUNT);
  OptAllowMultipleInstances  := iniFile.ReadBool(SECTION_GENERAL,   OPT_ALLOWMULTIPLEINSTANCES, DFLT_OPT_ALLOWMULTIPLEINSTANCES);
  OptDisplayToolbar          := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBAR,         DFLT_OPT_DISPLAYTOOLBAR);
  OptDisplayToolbarLarge     := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBARLARGE,    DFLT_OPT_DISPLAYTOOLBARLARGE);
  OptDisplayToolbarCaptions  := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYTOOLBARCAPTIONS, DFLT_OPT_DISPLAYTOOLBARCAPTIONS);
  OptDisplayStatusbar        := iniFile.ReadBool(SECTION_GENERAL,   OPT_DISPLAYSTATUSBAR,       DFLT_OPT_DISPLAYSTATUSBAR);
  OptAutoStartPortable       := iniFile.ReadBool(SECTION_GENERAL,   OPT_AUTOSTARTPORTABLE,      DFLT_OPT_AUTOSTARTPORTABLE);
  OptAdvancedMountDlg        := iniFile.ReadBool(SECTION_GENERAL,   OPT_ADVANCEDMOUNTDLG,       DFLT_OPT_ADVANCEDMOUNTDLG);
  OptDragDropFileType        := TDragDropFileType(iniFile.ReadInteger(SECTION_GENERAL, OPT_DRAGDROP, DFLT_OPT_DRAGDROP));
  OptRevertVolTimestamps     := iniFile.ReadBool(SECTION_GENERAL,   OPT_REVERTVOLTIMESTAMPS,    DFLT_OPT_REVERTVOLTIMESTAMPS);
  useDefaultDriveLetter      := iniFile.ReadString(SECTION_GENERAL, OPT_DEFAULTDRIVELETTER,     DFLT_OPT_DEFAULTDRIVELETTER);
  // #0 written as "#"
  OptDefaultDriveLetter := useDefaultDriveLetter[1];
  if (OptDefaultDriveLetter = '#') then
    begin
    OptDefaultDriveLetter := #0;
    end;
  OptShowPasswords           := iniFile.ReadBool(SECTION_GENERAL,   OPT_SHOWPASSWORDS,    DFLT_OPT_SHOWPASSWORDS);
  OptLanguageCode            := iniFile.ReadString(SECTION_GENERAL, OPT_LANGUAGECODE,     DFLT_OPT_LANGUAGECODE);

  OptPromptMountSuccessful       := iniFile.ReadBool(SECTION_CONFIRMATION, OPT_PROMPTMOUNTSUCCESSFUL,       DFLT_OPT_PROMPTMOUNTSUCCESSFUL);
  OptPromptDismountOnClose       := iniFile.ReadBool(SECTION_CONFIRMATION, OPT_PROMPTDISMOUNTONCLOSE,       DFLT_OPT_PROMPTDISMOUNTONCLOSE);
  OptPromptDriverShutdownOnClose := iniFile.ReadBool(SECTION_CONFIRMATION, OPT_PROMPTDRIVERSHUTDOWNONCLOSE, DFLT_OPT_PROMPTDRIVERSHUTDOWNONCLOSE);

  OptSystemTrayIconDisplay   := iniFile.ReadBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONDISPLAY, DFLT_OPT_SYSTEMTRAYICONDISPLAY);
  OptSystemTrayIconMinTo     := iniFile.ReadBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONMINTO,   DFLT_OPT_SYSTEMTRAYICONMINTO);
  OptSystemTrayIconCloseTo   := iniFile.ReadBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONCLOSETO, DFLT_OPT_SYSTEMTRAYICONCLOSETO);

  OptHKeyEnableDismount      := iniFile.ReadBool(SECTION_HOTKEY,   OPT_HOTKEYENABLEDISMOUNT,      DFLT_OPT_HOTKEYENABLEDISMOUNT);
  OptHKeyKeyDismount         := TextToShortCut(iniFile.ReadString(SECTION_HOTKEY, OPT_HOTKEYKEYDISMOUNT,         DFLT_OPT_HOTKEYKEYDISMOUNT));
  OptHKeyEnableDismountEmerg := iniFile.ReadBool(SECTION_HOTKEY,   OPT_HOTKEYENABLEDISMOUNTEMERG, DFLT_OPT_HOTKEYENABLEDISMOUNTEMERG);
  OptHKeyKeyDismountEmerg    := TextToShortCut(iniFile.ReadString(SECTION_HOTKEY, OPT_HOTKEYKEYDISMOUNTEMERG,    DFLT_OPT_HOTKEYKEYDISMOUNTEMERG));

  OptPostMountExe            := iniFile.ReadString(SECTION_AUTORUN,      OPT_POSTMOUNTEXE,          DFLT_OPT_POSTMOUNTEXE);
  OptPreDismountExe          := iniFile.ReadString(SECTION_AUTORUN,      OPT_PREDISMOUNTEXE,        DFLT_OPT_PREDISMOUNTEXE);
  OptPostDismountExe         := iniFile.ReadString(SECTION_AUTORUN,      OPT_POSTDISMOUNTEXE,       DFLT_OPT_POSTDISMOUNTEXE);
  OptPrePostExeWarn          := iniFile.ReadBool(SECTION_AUTORUN,        OPT_PREPOSTEXEWARN,        DFLT_OPT_PREPOSTEXEWARN);

  OptPKCS11Enable            := iniFile.ReadBool(SECTION_PKCS11,         OPT_PKCS11ENABLE,          DFLT_OPT_PKCS11ENABLE);
  OptPKCS11Library           := iniFile.ReadString(SECTION_PKCS11,       OPT_PKCS11LIBRARY,         DFLT_OPT_PKCS11LIBRARY);
  OptPKCS11AutoMount         := iniFile.ReadBool(SECTION_PKCS11,         OPT_PKCS11AUTOMOUNT,       DFLT_OPT_PKCS11AUTOMOUNT);
  OptPKCS11AutoMountVolume   := iniFile.ReadString(SECTION_PKCS11,       OPT_PKCS11AUTOMOUNTVOLUME, DFLT_OPT_PKCS11AUTOMOUNTVOLUME);
  OptPKCS11AutoDismount      := iniFile.ReadBool(SECTION_PKCS11,         OPT_PKCS11AUTODISMOUNT,    DFLT_OPT_PKCS11AUTODISMOUNT);

  OptMRUList.MaxItems := DFLT_OPT_MRUMAXITEMS;
  OptMRUList.Load(iniFile, SECTION_MRULIST);

end;

function TFreeOTFESettings.Save(): boolean;
var
  allOK: boolean;
  iniFile: TCustomINIFile;
begin
  allOK := FALSE;

  if (OptSaveSettings = slNone) then
    begin
    allOK := TRUE;
    end
  else
    begin
    iniFile := GetSettingsObj();
    if (iniFile <> nil) then
      begin
      try
        allOK := _Save(iniFile);

      finally
        iniFile.Free();
      end;
      end;
    end;

  if not(allOK) then
    begin
    allOK := FALSE;

    SDUMessageDlg(
               _('Your settings could not be saved.')+SDUCRLF+
                 SDUCRLF+
                 SDUParamSubstitute(_('Please ensure that you have suitable access rights in order to write to:'+SDUCRLF+
                       SDUCRLF+
                       '%1'),
                       [PrettyPrintSettingsFile(OptSaveSettings)]),
               mtError
              );
    end;

  Result := allOK;
end;

function TFreeOTFESettings._Save(iniFile: TCustomINIFile): boolean;
var
  allOK: boolean;
  useDefaultDriveLetter: char;
begin
  allOK := TRUE;

  try
    iniFile.WriteInteger(SECTION_GENERAL,     OPT_SETTINGSVERSION,           SETTINGS_V2);

    iniFile.WriteBool(SECTION_GENERAL,        OPT_EXPLOREAFTERMOUNT,         OptExploreAfterMount);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_ALLOWMULTIPLEINSTANCES,    OptAllowMultipleInstances);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBAR,            OptDisplayToolbar);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBARLARGE,       OptDisplayToolbarLarge);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYTOOLBARCAPTIONS,    OptDisplayToolbarCaptions);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_DISPLAYSTATUSBAR,          OptDisplayStatusbar);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_AUTOSTARTPORTABLE,         OptAutoStartPortable);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_ADVANCEDMOUNTDLG,          OptAdvancedMountDlg);
    iniFile.WriteInteger(SECTION_GENERAL,     OPT_DRAGDROP,                  ord(OptDragDropFileType));
    iniFile.WriteBool(SECTION_GENERAL,        OPT_REVERTVOLTIMESTAMPS,       OptRevertVolTimestamps);
    // #0 written as "#"
    useDefaultDriveLetter := OptDefaultDriveLetter;
    if (OptDefaultDriveLetter = #0) then
      begin
      useDefaultDriveLetter := '#';
      end;
    iniFile.WriteString(SECTION_GENERAL,      OPT_DEFAULTDRIVELETTER,        useDefaultDriveLetter);
    iniFile.WriteBool(SECTION_GENERAL,        OPT_SHOWPASSWORDS,             OptShowPasswords);
    iniFile.WriteString(SECTION_GENERAL,      OPT_LANGUAGECODE,              OptLanguageCode);

    iniFile.WriteBool(SECTION_CONFIRMATION,  OPT_PROMPTMOUNTSUCCESSFUL,       OptPromptMountSuccessful);
    iniFile.WriteBool(SECTION_CONFIRMATION,  OPT_PROMPTDISMOUNTONCLOSE,       OptPromptDismountOnClose);
    iniFile.WriteBool(SECTION_CONFIRMATION,  OPT_PROMPTDRIVERSHUTDOWNONCLOSE, OptPromptDriverShutdownOnClose);

    iniFile.WriteBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONDISPLAY,     OptSystemTrayIconDisplay);
    iniFile.WriteBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONMINTO,       OptSystemTrayIconMinTo);
    iniFile.WriteBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONCLOSETO,     OptSystemTrayIconCloseTo);

    iniFile.WriteBool(SECTION_HOTKEY,         OPT_HOTKEYENABLEDISMOUNT,      OptHKeyEnableDismount);
    iniFile.WriteString(SECTION_HOTKEY,       OPT_HOTKEYKEYDISMOUNT,         ShortCutToText(OptHKeyKeyDismount));
    iniFile.WriteBool(SECTION_HOTKEY,         OPT_HOTKEYENABLEDISMOUNTEMERG, OptHKeyEnableDismountEmerg);
    iniFile.WriteString(SECTION_HOTKEY,       OPT_HOTKEYKEYDISMOUNTEMERG,    ShortCutToText(OptHKeyKeyDismountEmerg));

    iniFile.WriteString(SECTION_AUTORUN,      OPT_POSTMOUNTEXE,              OptPostMountExe);
    iniFile.WriteString(SECTION_AUTORUN,      OPT_PREDISMOUNTEXE,            OptPreDismountExe);
    iniFile.WriteString(SECTION_AUTORUN,      OPT_POSTDISMOUNTEXE,           OptPostDismountExe);
    iniFile.WriteBool(SECTION_AUTORUN,        OPT_PREPOSTEXEWARN,            OptPrePostExeWarn);

    iniFile.WriteBool(SECTION_PKCS11,         OPT_PKCS11ENABLE,              OptPKCS11Enable);
    iniFile.WriteString(SECTION_PKCS11,       OPT_PKCS11LIBRARY,             OptPKCS11Library);
    iniFile.WriteBool(SECTION_PKCS11,         OPT_PKCS11AUTOMOUNT,           OptPKCS11AutoMount);
    iniFile.WriteString(SECTION_PKCS11,       OPT_PKCS11AUTOMOUNTVOLUME,     OptPKCS11AutoMountVolume);
    iniFile.WriteBool(SECTION_PKCS11,         OPT_PKCS11AUTODISMOUNT,        OptPKCS11AutoDismount);

    OptMRUList.Save(iniFile, SECTION_MRULIST);
    
  except
    on E:Exception do
      begin
      allOK := FALSE;
      end;
  end;

  Result := allOK;
end;

function TFreeOTFESettings.PrettyPrintSettingsFile(loc: TSettingsSaveLocation): string;
var
  retval: string;
begin
  retval := '';

  case loc of
    slNone:
      begin
      // Do nothing
      end;

    slExeDir,
    slProfile:
      begin
      retval := GetSettingsFilename(loc);
      end;

    slRegistry:
      begin
      retval := REGISTRY_SETTINGS_LOCATION;
      end;

    else
      begin
      SDUMessageDlg(UNKNOWN_SETTINGS_LOCATION, mtError);
      end;

    end;

  Result := retval;
end;


function TFreeOTFESettings.DestroySettingsFile(loc: TSettingsSaveLocation): boolean;
var
  filename: string;
  retval: boolean;
  registry: TRegistry;
begin
  retval := TRUE;

  case loc of
    slNone:
      begin
      // Do nothing; nothing to delete
      end;

    slExeDir,
    slProfile:
      begin
      filename := GetSettingsFilename(loc);
      if (filename <> '') then
        begin
        retval := DeleteFile(filename);
        end;
      end;

    slRegistry:
      begin
      registry:= TRegistry.Create();
      try
        registry.RootKey := HKEY_CURRENT_USER;
        retval := registry.DeleteKey(REGISTRY_SETTINGS_LOCATION);
      finally
        registry.Free();
      end;
      end;

    else
      begin
      SDUMessageDlg(UNKNOWN_SETTINGS_LOCATION, mtError);
      end;

    end;

  Result := retval;
end;

function TFreeOTFESettings.GetSettingsFilename(loc: TSettingsSaveLocation): string;
var
  iniFilenameOnly: string;
  filenameAndPath: string;
begin
  filenameAndPath := '';

  iniFilenameOnly := ChangeFileExt(ExtractFilename(ParamStr(0)), '.ini');

  case loc of
    slNone,
    slRegistry:
      begin
      // Do nothing; already set to empty string
      end;

    slExeDir:
      begin
      filenameAndPath := ExtractFilePath(ParamStr(0)) + iniFilenameOnly;
      end;

    slProfile:
      begin
      filenameAndPath := SDUGetSpecialFolderPath(CSIDL_PERSONAL)+
                         '\'+
                         iniFilenameOnly
      end;

    else
      begin
      SDUMessageDlg(UNKNOWN_SETTINGS_LOCATION, mtError);
      end;

    end;

  Result := filenameAndPath;
end;

// Identify where user settings are stored
function TFreeOTFESettings.IdentifyWhereSettingsStored(): TSettingsSaveLocation;
var
  sl: TSettingsSaveLocation;
  filename: string;
  retval: TSettingsSaveLocation;
  checkLocation: TSettingsSaveLocation;
  registry: TRegistry;
begin
  retval := slNone;

  for sl := low(SettingsSaveLocationSearchOrder) to high(SettingsSaveLocationSearchOrder) do
    begin
    checkLocation := SettingsSaveLocationSearchOrder[sl];

    if (
        (checkLocation = slExeDir) or
        (checkLocation = slProfile)
       ) then
      begin
      filename := GetSettingsFilename(checkLocation);
      if (
          (filename <> '') and
          FileExists(filename)
         ) then
        begin
        retval := checkLocation;
        break;
        end;
      end
    else if (checkLocation = slRegistry) then
      begin
      // Test if registry key exists
      registry:= TRegistry.Create();
      try
        registry.RootKey := HKEY_CURRENT_USER;
        if registry.OpenKeyReadOnly(REGISTRY_SETTINGS_LOCATION) then
          begin
          retval := checkLocation;
          break;
          end;
      finally
        registry.Free();
      end;

      end;

    end;

  Result := retval;
end;

procedure TFreeOTFESettings.Assign(copyFrom: TFreeOTFESettings);
var
  iniFile: TINIFile;
begin
  // lplp - disabled for now.
  // lplp  - only used by options dialog, which should just save/restore internally; *shouldn't* need this function, but included for completeness
  exit;

  iniFile := TINIFile.Create('');
  try
    copyFrom._Save(iniFile);
    self._Load(iniFile);
  finally
    iniFile.Free();
  end;

end;

// Important: The *caller* is responsible for freeing off the object returned
// Retuns: A TCustomIniFile that can be used for saving/loading settings.
// This will return NIL if there's a problem (e.g. the .INI file can't be
// created) 
function TFreeOTFESettings.GetSettingsObj(): TCustomIniFile;
var
  filename: string;
  retval: TCustomIniFile;
begin
  retval := nil;

  if (OptSaveSettings = slNone) then
    begin
    // Create "ini file" with blank filename; all calls to read from this will
    // fail; resulting in the default values being returned/used
    retval := TINIFile.Create('');
    end
  else if (
           (OptSaveSettings = slExeDir) or
           (OptSaveSettings = slProfile)
          ) then
    begin
    filename := GetSettingsFilename(OptSaveSettings);
    try
      retval := TINIFile.Create(filename);
    except
      // Problem - e.g. can't create .INI file
      retval := nil;
    end;

    end
  else if (OptSaveSettings = slRegistry) then
    begin
      retval := TRegistryINIFile.Create(REGISTRY_SETTINGS_LOCATION);
    end;

  Result := retval;
end;

END.


